# to do: errors

import socket
import random

class Error(Exception):
    def __init__(self, message):
        super().__init__(message)

# RSA
def modular_inverse(a, n):
    s0, s1 = 1, 0
    r0, r1 = a, n

    while r1 > 0:
        q = r0 // r1
        r0, r1 = r1, r0 - (q*r1)
        s0, s1 = s1, s0 - (q*s1)

    return s0

def sieve_of_eratosthenes(n):
    primes = []
    is_prime = [True] * (n + 1)
    p = 2
    while p * p <= n:
        if is_prime[p]:
            for i in range(p * p, n + 1, p):
                is_prime[i] = False
        p += 1
    for p in range(2, n + 1):
        if is_prime[p]:
            primes.append(p)
    return primes

def random_prime(n):
    return random.choice(sieve_of_eratosthenes(n))


# Network
_server = None
_connection = None

def get_address():
    hostname = socket.gethostname()
    return socket.gethostbyname(hostname)

def create_connection(role, comp_a_ip):
    global _connection, _server
    PORT = 3427

    if role == 'a':
        HOST = '0.0.0.0'

        server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        for _ in range(5):
            try:
                server.bind((HOST, PORT))
                break
            except socket.error:
                bind += random.randint(1, 50)
        else:
            raise Error('Open port not found')
        server.listen()

        conn, addr = server.accept()
        print(f'Connected by {addr}')

        _server = server
        _connection = conn
    elif role == 'b':
        HOST = comp_a_ip

        client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        for _ in range(5):
            try:
                client.connect((HOST, PORT))
                break
            except socket.error:
                bind += random.randint(1, 50)
        else:
            raise Error('Open port not found')
        print('Connected')

        _connection = client
    
def send(message):
    _connection.sendall(message.encode())

def receive(size=2048):
    data = _connection.recv(size).decode()
    return data

def close_connection(connection):
    _connection.close()
    if _server: _server.close()




